@@ -13,10 +13,25 @@
 # limitations under the License.
 
 """Exceptions raised by PyMongo."""
-from typing import Any, Iterable, List, Mapping, Optional, Sequence, Tuple, Union
+from __future__ import annotations
+
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Iterable,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+)
 
 from bson.errors import InvalidDocument
 
+if TYPE_CHECKING:
+    from pymongo.typings import _DocumentOut
+
 try:
     # CPython 3.7+
     from ssl import SSLCertVerificationError as _CertificateError
@@ -33,7 +48,7 @@ class PyMongoError(Exception):
     """Base class for all PyMongo exceptions."""
 
     def __init__(self, message: str = "", error_labels: Optional[Iterable[str]] = None) -> None:
-        super(PyMongoError, self).__init__(message)
+        super().__init__(message)
         self._message = message
         self._error_labels = set(error_labels or [])
 
@@ -44,11 +59,11 @@ class PyMongoError(Exception):
         """
         return label in self._error_labels
 
-    def _add_error_label(self, label):
+    def _add_error_label(self, label: str) -> None:
         """Add the given label to this error."""
         self._error_labels.add(label)
 
-    def _remove_error_label(self, label):
+    def _remove_error_label(self, label: str) -> None:
         """Remove the given label from this error."""
         self._error_labels.discard(label)
 
@@ -105,7 +120,7 @@ class AutoReconnect(ConnectionFailure):
         if errors is not None:
             if isinstance(errors, dict):
                 error_labels = errors.get("errorLabels")
-        super(AutoReconnect, self).__init__(message, error_labels)
+        super().__init__(message, error_labels)
         self.errors = self.details = errors or []
 
 
@@ -123,9 +138,9 @@ class NetworkTimeout(AutoReconnect):
         return True
 
 
-def _format_detailed_error(message, details):
+def _format_detailed_error(message: str, details: Optional[Union[Mapping[str, Any], List]]) -> str:
     if details is not None:
-        message = "%s, full error: %s" % (message, details)
+        message = f"{message}, full error: {details}"
     return message
 
 
@@ -148,9 +163,7 @@ class NotPrimaryError(AutoReconnect):
     def __init__(
         self, message: str = "", errors: Optional[Union[Mapping[str, Any], List]] = None
     ) -> None:
-        super(NotPrimaryError, self).__init__(
-            _format_detailed_error(message, errors), errors=errors
-        )
+        super().__init__(_format_detailed_error(message, errors), errors=errors)
 
 
 class ServerSelectionTimeoutError(AutoReconnect):
@@ -191,15 +204,13 @@ class OperationFailure(PyMongoError):
         error_labels = None
         if details is not None:
             error_labels = details.get("errorLabels")
-        super(OperationFailure, self).__init__(
-            _format_detailed_error(error, details), error_labels=error_labels
-        )
+        super().__init__(_format_detailed_error(error, details), error_labels=error_labels)
         self.__code = code
         self.__details = details
         self.__max_wire_version = max_wire_version
 
     @property
-    def _max_wire_version(self):
+    def _max_wire_version(self) -> Optional[int]:
         return self.__max_wire_version
 
     @property
@@ -290,10 +301,10 @@ class BulkWriteError(OperationFailure):
     .. versionadded:: 2.7
     """
 
-    details: Mapping[str, Any]
+    details: _DocumentOut
 
-    def __init__(self, results: Mapping[str, Any]) -> None:
-        super(BulkWriteError, self).__init__("batch op errors occurred", 65, results)
+    def __init__(self, results: _DocumentOut) -> None:
+        super().__init__("batch op errors occurred", 65, results)
 
     def __reduce__(self) -> Tuple[Any, Any]:
         return self.__class__, (self.details,)
@@ -331,8 +342,6 @@ class InvalidURI(ConfigurationError):
 class DocumentTooLarge(InvalidDocument):
     """Raised when an encoded document is too large for the connected server."""
 
-    pass
-
 
 class EncryptionError(PyMongoError):
     """Raised when encryption or decryption fails.
@@ -344,7 +353,7 @@ class EncryptionError(PyMongoError):
     """
 
     def __init__(self, cause: Exception) -> None:
-        super(EncryptionError, self).__init__(str(cause))
+        super().__init__(str(cause))
         self.__cause = cause
 
     @property
@@ -359,7 +368,27 @@ class EncryptionError(PyMongoError):
         return False
 
 
+class EncryptedCollectionError(EncryptionError):
+    """Raised when creating a collection with encrypted_fields fails.
+
+    .. versionadded:: 4.4
+    """
+
+    def __init__(self, cause: Exception, encrypted_fields: Mapping[str, Any]) -> None:
+        super().__init__(cause)
+        self.__encrypted_fields = encrypted_fields
+
+    @property
+    def encrypted_fields(self) -> Mapping[str, Any]:
+        """The encrypted_fields document that allows inferring which data keys are *known* to be created.
+
+        Note that the returned document is not guaranteed to contain information about *all* of the data keys that
+        were created, for example in the case of an indefinite error like a timeout. Use the `cause` property to
+        determine whether a definite or indefinite error caused this error, and only rely on the accuracy of the
+        encrypted_fields if the error is definite.
+        """
+        return self.__encrypted_fields
+
+
 class _OperationCancelled(AutoReconnect):
     """Internal error raised when a socket operation is cancelled."""
-
-    pass
